#!/bin/awk -f
## Jan 12, 2006
## Quality control of MASS data
## bad background (>3%), Chi2>100
## Use as: check.awk mass-file [ >log-file]
BEGIN{
#   print "Setting the parameters"
    good = 0; backd=1; fluxd=1; tgood=0;  # flag to mark good data
    ntot = 0; ngood=0; nflux = 0;   # number of total and good measurements
    for (i=1;i<11;i++) {res[i]=0.}  # average residuals
    for (i=1;i<4;i++) {rat[i]=0.}   # average flux ratios
    chi2av = 0.; fseeav=0; theta0av=0.; tauav=0.;
### The quality criteria go here, change as appropriate #########
    Dfluxmin = 100.    # Min. useful flux in the D channel
    Bckmax = 0.03      # Max. admissible background fraction
    Chi2max = 100      # Max. Chi2 measure
    Fluxvarmax = 0.01  # Max. flux variability (cloud control)
    Residmax = 0.06    # Max. average residuals on 10 indices
    Ratmax = 0.05      # Max. deviation of flux ratio from nominal
    Effmin = 0.6       # Min. fraction of valid data
### The instrument-specific parameters go here, change for each unit
    CDrat = 0.61  # Nominal flux ratio C/D
    BCrat = 0.30  # Nominal flux ratio B/C
    ABrat = 0.59  # Nominal flux ratio A/B 
}
{ 
    if ($1=="P" &&  substr($4,12,10)=="Background") {backd = $8}

# Check of the flux value and variability
    if ($1=="F") {
      fluxd = $10; if (fluxd==0) {fluxd=1};
      if (backd/fluxd<Bckmax && fluxd>Dfluxmin && $11<Fluxvarmax) {good = 1} else {good = 0}

# Compute the average flux ratios
    if ($10>0 && good==1){ nflux +=1; rat[1]+=$8/$10;  rat[2]+=$6/$8;  rat[3]+=$4/$6; }
    }
    if ($1=="A") {fsee = $4; theta0 = $16; tau0 = $20}

# Check the Chi2
    if ($1=="T") {chi2 = $6;  if(chi2>Chi2max) {good=0}; }

# Compute average residuals and count points
      if ($1=="R" && $4=="X") { ntot +=1;
	if (good ==1) {ngood +=1;
	for (i=1;i<11;i++) {res[i] += $(4+i)} }
        fseeav += fsee;  theta0av+=theta0; tauav+=tau0;
        chi2av += chi2;
      }
#      print "Good points: ", ngood
}     
# 
END{
# Give general results of quality check
#
  if (ngood <10) {print ARGV[1]" !! Less than 10 valid data"}
  else {
   if ( ngood/ntot < Effmin) {FL="!!"} else {FL="OK"}
   print ARGV[1],FL,"Total ",ntot,"Good", ngood, "Efficiency ",ngood/ntot
# Check flux ratios
   for (i=1;i<4;i++) {rat[i]=rat[i]/nflux} 
   FL="OK"
   if (rat[1]>CDrat+Ratmax || rat[1]<CDrat-Ratmax ) {FL="!!"}
   if (rat[2]>BCrat+Ratmax || rat[2]<BCrat-Ratmax ) {FL="!!"}
   if (rat[3]>ABrat+Ratmax || rat[3]<ABrat-Ratmax ) {FL="!!"}
   printf "%s %s %s %6.3f %6.3f  %6.3f  \n", ARGV[1],FL," Flux ratios: ",rat[1],rat[2],rat[3]

# Check the average residuals
   FL="OK"
   for (i=1;i<11;i++) {res[i]=res[i]/ngood} 
   for (i=1;i<11;i++) {if (res[i] > Residmax || res[i] < -Residmax) {FL="!!"}}
   printf "%s %s %s", ARGV[1],FL," Residuals" 
   for (i=1;i<11;i++) {printf "%7.3f", res[i]} 
   printf "\n"
# Output the average atmospheric parameters
     printf "%s %6.3f %6.3f  %6.3f  %6.3f \n", ARGV[1]" Fsee Theta0,Tau0,Chi2: ",fseeav/ngood, theta0av/ngood, tauav/ngood, chi2av/ngood
     }
      }












